package com.usky.issue.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.StringUtils;
import com.usky.issue.domain.*;
import com.usky.issue.mapper.SpRtu2017Mapper;
import com.usky.issue.service.SpOwnerService;
import com.usky.issue.service.SpRtu2017Service;
import com.usky.common.mybatis.core.AbstractCrudService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author fu
 * @since 2023-12-13
 */
@Service
@DS("jdxf")
public class SpRtu2017ServiceImpl extends AbstractCrudService<SpRtu2017Mapper, SpRtu2017> implements SpRtu2017Service {
    @Autowired
    private SpOwnerService spOwnerService;

    /**
     * 设备状态统计
     * @param requestBody
     * @return
     */
    @Override
    public List<Map<String, Object>> statusCount(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();
        Integer currentPage = (Integer) requestVO.get("current");
        Integer pageSize = (Integer) requestVO.get("size");
        int startIndex = (currentPage - 1) * pageSize;
        QueryWrapper<SpRtu2017> queryWrapper = Wrappers.query();
        queryWrapper.select("DATE_FORMAT(time, '%Y-%m-%d %H:%i:%s') AS alarm_time", "data2 AS alarm_type", "data4 AS alarm_point", "device_code")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "' AND dwtype = 6");
        queryWrapper.last("LIMIT " + pageSize + " OFFSET " + startIndex)
                .orderByDesc("id");
        List<Map<String, Object>> deviceStatusList = this.listMaps(queryWrapper);
        QueryWrapper<SpOwner> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("owner_code","unitinfo")
                .eq("company",companyCode)
                .eq("dwtype",6);
        List<Map<String, Object>> deviceInfoList = spOwnerService.listMaps(queryWrapper1);
        if(CollectionUtils.isNotEmpty(deviceStatusList)){
            for (int i=0;i<deviceStatusList.size();i++){
                if(CollectionUtils.isNotEmpty(deviceInfoList)){
                    for (int j=0;j<deviceInfoList.size();j++){
                        if(deviceStatusList.get(i).get("device_code").equals(deviceInfoList.get(j).get("owner_code"))){
                            deviceStatusList.get(i).put("device_address",deviceInfoList.get(j).get("unitinfo"));
                            break;
                        }
                    }
                }
            }
        }
        return deviceStatusList;
    }

    /**
     * 设备查询
     * @param requestBody
     * @return
     */
    public List<Map<String, Object>> typeList(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();
        QueryWrapper<SpRtu2017> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("COUNT(CASE WHEN clzt = 0 THEN 1 ELSE NULL END) AS untreated", "COUNT(CASE WHEN clzt = 1 THEN 1 ELSE NULL END) AS processed")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "' AND dwtype = 6");
        List<Map<String, Object>> deviceStatusList1 = this.listMaps(queryWrapper1);
        return deviceStatusList1;
    }

    /**
     * 设备数量统计
     * @param requestBody
     * @return
     */
    public Integer countDevice(String requestBody) {
        JSONObject requestVO = JSONObject.parseObject(requestBody);
        String companyCode = requestVO.get("companyCode").toString();

        QueryWrapper<SpRtu2017> queryWrapper1 = Wrappers.query();
        queryWrapper1.select("DISTINCT device_code")
                .inSql("device_code", "SELECT owner_code FROM sp_owner WHERE company = '" + companyCode + "'");
        List<SpRtu2017> SpRtu2017 = this.list(queryWrapper1);

        Integer deviceCount = SpRtu2017.size();

        return deviceCount;
    }

    /**
     * 告警信息更新
     * @return
     */
    @Override
    public List<SpRtu2017> getOwners() {
        List<String> codes = spOwnerService.getOwners(spOwnerService.getDeviceCodes())
                .stream()
                .filter(spOwner -> spOwner.getDwtype()==6)
                .map(SpOwner::getOwnerCode)
                .collect(Collectors.toList());
        if (!codes.isEmpty()) {

        }
        return baseMapper.selectList(Wrappers.<SpRtu2017>query()
                .select("device_code", "time", "data2","data4","clr",
                        "IF(clsj = '0000-00-00 00:00:00', '1997-01-01 00:00:00', clsj) AS clsj",
                        "clnr", "clzt", "clwb")
                .in("device_code", codes)
        );
    }

}
